/* global ActionBase */

class AudioAction extends ActionBase {
  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
  }

  #flattenPlaylists = (list) => {
    let playlists = [];

    list.forEach((item) => {
      if (item.type === 'playlist') {
        playlists.push(item);
      } else if (item.type === 'group') {
        playlists = playlists.concat(this.#flattenPlaylists(item.children));
      }
    });

    return playlists;
  };

  async onSendToPlugin(json) {
    if (typeof json.payload.sdpi_collection === 'undefined' || json.payload.sdpi_collection.key !== 'audioPlaylistSelector') {
      return;
    }

    const playlistID = json.payload.sdpi_collection.value;

    try {
      const selectedPlaylistAudioItems = await this.apiManager.getAudioItemsInPlaylist(playlistID);
      ActionBase.sendToPropertyInspector(json.context, { selectedPlaylistAudioItems }, json.action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(json.context, { selectedPlaylistAudioItems: null }, json.action);
    }
  }

  async pollForData(context, action) {
    try {
      const playlists = await this.apiManager.getAudioPlaylists();
      ActionBase.sendToPropertyInspector(context, { availablePlaylists: this.#flattenPlaylists(playlists) }, action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(context, { availablePlaylists: null }, action);
    }

    super.pollForData(context, action);
  }

  async performAction(settings) {
    const selectedPlaylistID = settings.audioPlaylistSelector;
    const selectedItemID = settings.audioItemSelector;
    if (typeof selectedPlaylistID === 'undefined') {
      throw new Error('audioPlaylistSelector not defined in settings');
    }
    if (typeof selectedItemID !== 'undefined' && selectedItemID !== 'null') {
      await this.apiManager.triggerAudio(selectedPlaylistID, selectedItemID);
    } else {
      await this.apiManager.triggerAudio(selectedPlaylistID, null);
    }
  }
}

if (typeof module !== 'undefined') {
  module.exports = AudioAction;
}
