/* global ActionBase */

class GroupAction extends ActionBase {
  /**
     * This is a description of the Group Action.
     *
     * @param {*} apiManager - Some description of this param
     */
  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
  }

  async onDidReceiveSettings(json) {
    GroupAction.#setTitleForSelectedGroup(json.payload.settings, json.context);
    super.onDidReceiveSettings(json);
  }

  onWillAppear(json) {
    GroupAction.#setTitleForSelectedGroup(json.payload.settings, json.context);
    super.onWillAppear(json);
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;

    if (typeof settings['icon-state'] === 'undefined') {
      settings['icon-state'] = 'trigger';
      didUpdateSettings = true;
    }

    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async pollForData(context, action) {
    try {
      const groups = await this.apiManager.getGroups();
      ActionBase.sendToPropertyInspector(context, { availableGroups: groups }, action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(context, { availableGroups: null }, action);
    }

    super.pollForData(context, action);
  }

  async performAction(settings) {
    const group = settings.groupSelector;
    if (typeof group === 'undefined') {
      throw new Error('groupSelector not defined in settings');
    }
    await this.apiManager.triggerGroup(group);
  }

  // *** Private Methods ***

  /**
       * Sets the title to the group name. User can later override this by entering a title in the title field.
       * @param {Object} settings - The settings to use when determining the title.
       * @param {string} context - The context UUID of this action instance.
       */
  static #setTitleForSelectedGroup = (settings, context) => {
    const { selectedGroup } = settings;
    if (typeof selectedGroup !== 'undefined') {
      ActionBase.setTitle(context, selectedGroup.id.name);
    }
  };
}

if (typeof module !== 'undefined') {
  module.exports = GroupAction;
}
