/* global ActionBase */

class MacroAction extends ActionBase {
  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
  }

  async pollForData(context, action) {
    try {
      const macros = await this.apiManager.getMacros();
      ActionBase.sendToPropertyInspector(context, { availableMacros: macros }, action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(context, { availableMacros: null }, action);
    }

    super.pollForData(context, action);
  }

  async performAction(settings) {
    const macroId = settings.macroSelector;
    if (typeof macroId === 'undefined') {
      throw new Error('macroSelector not defined in settings');
    }
    await this.apiManager.triggerMacro(macroId);
  }
}

if (typeof module !== 'undefined') {
  module.exports = MacroAction;
}
