/* global ActionBase */

class VideoInputAction extends ActionBase {
  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;

    if (typeof settings['icon-state'] === 'undefined') {
      settings['icon-state'] = 'trigger';
      didUpdateSettings = true;
    }

    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async pollForData(context, action) {
    try {
      const inputs = await this.apiManager.getVideoInputs();
      ActionBase.sendToPropertyInspector(context, { availableVideoInputs: inputs }, action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(context, { availableVideoInputs: null }, action);
    }

    super.pollForData(context, action);
  }

  async performAction(settings) {
    const videoInputId = settings.videoInputSelector;
    if (typeof videoInputId === 'undefined') {
      throw new Error('videoInputSelector not defined in settings');
    }
    await this.apiManager.triggerVideoInput(videoInputId);
  }
}

if (typeof module !== 'undefined') {
  module.exports = VideoInputAction;
}
