/* eslint-disable no-use-before-define */
/* global $SD, Utils, $ */
/* eslint-disable no-extra-boolean-cast */
/* eslint-disable no-else-return */

/**
 * This example contains a working Property Inspector, which already communicates
 * with the corresponding plugin throug settings and/or direct messages.
 * If you want to use other control-types, we recommend copy/paste these from the
 * PISamples demo-library, which already contains quite some example DOM elements
 */

/**
 * First we declare a global variable, which change all elements behaviour
 * globally. It installs the 'onchange' or 'oninput' event on the HTML controls and fiels.
 *
 * Change this, if you want interactive elements act on any modification (oninput),
 * or while their value changes 'onchange'.
 */

const onchangeevt = 'onchange'; // 'oninput';

/**
 * Since the Property Inspector is instantiated every time you select a key
 * in Stream Deck software, we can savely cache our settings in a global variable.
 */

let settings;
let globalSettings;

/**
 * The 'connected' event is the first event sent to Property Inspector, after it's instance
 * is registered with Stream Deck software. It carries the current websocket, settings,
 * and other information about the current environmet in a JSON object.
 * You can use it to subscribe to events you want to use in your plugin.
 */

/** This injects the global settings that are included in every action of the plugin into the property inspectors */
document.getElementById('global-settings-placeholder').innerHTML = `
    <div class="sdpi-item">
        <div class="sdpi-item-label">Color</div>
            <input type="color" id="icon-background-color" class="sdpi-item-value" value="#2D2D2D">
            <label for="radio-dark-icon" class="sdpi-item-label max10">Icon</label>
            <select class="sdpi-item-value select" id="icon-tint">
              <option value="lightIcon">Light</option>
              <option value="darkIcon">Dark</option>
            </select>
        </div>
    
        <!-- Divider line -->
        <hr>
    
        <div class="sdpi-item">
            <div class="sdpi-item-label" id="connection-label">Connection</div>
            <select class="sdpi-item-value select connection-selector" id="connection-select"></select>
            <button onclick="deleteSelectedConnection();" class="select-modifier-button">
              <img class="delete-image"/>
            </button>
        </div>
    </div>
`;

document.getElementById('communication-alert').innerHTML = `
    <details class = "message caution">
      <summary>Unable to communicate with ProPresenter</summary>
      <p>Check your connection settings or choose a<br>different connection.</p>
    </details>
`;

document.addEventListener('saveConnection', saveConnection);

$(document).ready(() => {
  $('#communication-alert').hide();

  $('#connection-select').change(() => {
    const socketAddress = $('#connection-select').val();

    switch (socketAddress) {
      case 'add-connection':
        // Open setup window
        window.open(`../setupWindows/ConnectionSetup.html?uuid=${$SD.uuid}`);
        // Load connections so that the selected connection will be reselected in the dropdown
        loadConnections();
        break;
      case 'no-connections':
        // Do Nothing
        break;
      default:
        // A connection was selected, so save it as the selected connection
        globalSettings.selectedConnection = globalSettings.connections[socketAddress];
        saveGlobalSettings();
    }
  });
});

// eslint-disable-next-line no-unused-vars
function deleteSelectedConnection() {
  const filteredConnections = {};

  let prevConnection; // Keep track of previous connection when looping

  Object.entries(globalSettings.connections).forEach(([key, value]) => {
    if (globalSettings.selectedConnection && key === globalSettings.selectedConnection.socketAddress) {
      // This is the connection we want to delete.
      if (prevConnection) {
        // If there is a previous connection in the list, select that.
        globalSettings.selectedConnection = prevConnection;
      } else {
        // Otherwise set to null so it can get set to the next connection if there is one.
        globalSettings.selectedConnection = null;
      }
    } else {
      filteredConnections[key] = value;
      if (globalSettings.selectedConnection === null) {
        // Select a connection if one isn't selected.
        globalSettings.selectedConnection = value;
      }
    }

    prevConnection = value;
  });

  globalSettings.connections = filteredConnections;

  saveGlobalSettings();
  $SD.api.getGlobalSettings($SD.uuid);
}

// Called by setup window when user adds a new connection
function saveConnection(inEvent) {
  const newConnection = {
    name: inEvent.detail.name,
    socketAddress: `${inEvent.detail.ip}:${inEvent.detail.port}`,
  };

  globalSettings.connections[newConnection.socketAddress] = newConnection;
  globalSettings.selectedConnection = newConnection;
  saveGlobalSettings();

  $SD.api.getGlobalSettings($SD.uuid);
}

function loadConnections() {
  const connectionOptionElements = $('.connection-option');
  const selector = $('#connection-select');

  // Remove all connections from dropdown
  connectionOptionElements.each((_, element) => {
    element.remove();
  });

  const connections = Object.values(globalSettings.connections);
  const { selectedConnection } = globalSettings;

  if (connections.length > 0 && typeof selectedConnection !== 'undefined') {
    // Add all incoming connections
    connections.forEach((connection) => {
      const connectionOption = $(`<option class="connection-option" value="${connection.socketAddress}")>
        ${connection.name}</option>`)
        .appendTo(selector);

      if (connection.socketAddress === selectedConnection.socketAddress) {
        connectionOption.prop('selected', true);
      }
    });
  } else {
    // Add the "No connections" option if there aren't any connections
    selector.append('<option id="no-connections" class="connection-option" value="no-connections")>No connections</option>');
  }

  // Add the "Add new" option
  selector.append('<option id="add-connection" class="connection-option" value="add-connection")>Add new</option>');
  $SD.emit('Connections-Updated');
}

$SD.on('connected', (jsn) => {
  settings = Utils.getProp(jsn, 'actionInfo.payload.settings', false);
  if (settings) {
    updateUI(settings);
  }

  // Get global settings in order to update the connections dropdown when first loading.
  $SD.api.getGlobalSettings($SD.uuid);
});

const updateUI = (pl) => {
  Object.keys(pl).forEach((e) => {
    if (e && e !== '') {
      const foundElement = document.querySelector(`#${e}`);
      if (foundElement && foundElement.type !== 'file') {
        foundElement.value = pl[e];
        const maxl = foundElement.getAttribute('maxlength') || 50;
        const labels = document.querySelectorAll(`[for='${foundElement.id}']`);
        if (labels.length) {
          labels.forEach((label) => {
            label.textContent = maxl ? `${foundElement.value.length}/${maxl}` : `${foundElement.value.length}`;
          });
        }
      }
    }
  });

  // Called when the UI is loaded initially.
  $SD.emit('UIUpdated');
};

/**
  * Below are a bunch of helpers to make your DOM interactive
  * The will cover changes of the most common elements in your DOM
  * and send their value to the plugin on a change.
  * To accomplish this, the 'handleSdpiItemChange' method tries to find the
  * nearest element 'id' and the corresponding value of the element(along with
  * some other information you might need) . It then puts everything in a
  * 'sdpi_collection', where the 'id' will get the 'key' and the 'value' will get the 'value'.
  *
  * In the plugin you just need to listen for 'sdpi_collection' in the sent JSON.payload
  * and save the values you need in your settings (or StreamDeck-settings for persistence).
  *
  * In this template those key/value pairs are saved automatically persistently to StreamDeck.
  * Open the console in the remote debugger to inspect what's getting saved.
  *
  */
function saveSettings(keyValuePair) {
  if (typeof keyValuePair !== 'object'
  || typeof keyValuePair.key === 'undefined'
  || typeof keyValuePair.value === 'undefined') {
    return;
  }
  settings[keyValuePair.key] = keyValuePair.value;
  $SD.api.setSettings($SD.uuid, settings);
}

/**
 * Saves the cached global settings to the Stream Deck
*/
function saveGlobalSettings() {
  $SD.api.setGlobalSettings($SD.uuid, globalSettings);
}

$SD.on('didReceiveGlobalSettings', (json) => {
  // Update our cached settings
  globalSettings = json.payload.settings;

  // Ensure we have a connections object
  if (typeof globalSettings.connections === 'undefined') {
    globalSettings.connections = {};
    saveGlobalSettings();
  }

  loadConnections();
});

/**
 * Something in the PI changed:
 * either you clicked a button, dragged a slider or entered some text
 *
 * The 'piDataChanged' event is sent, if data-changes are detected.
 * The changed data are collected in a JSON structure
 *
 * It looks like this:
 *
 *  {
 *      checked: false
 *      group: false
 *      index: 0
 *      key: "mynameinput"
 *      selection: []
 *      value: "Elgato"
 *  }
 *
 * If you set an 'id' to an input-element, this will get the 'key' of this object.
 * The input's value will get the value.
 * There are other fields (e.g.
 *      - 'checked' if you clicked a checkbox
 *      - 'index', if you clicked an element within a group of other elements
 *      - 'selection', if the element allows multiple-selections
 * )
 *
 * Please note:
 * the template creates this object for the most common HTML input-controls.
 * This is a convenient way to start interacting with your plugin quickly.
 *
 */
$SD.on('piDataChanged', (returnValue) => {
  /* SAVE THE VALUE TO SETTINGS */
  saveSettings(returnValue);

  /* SEND THE VALUES TO PLUGIN */
  sendValueToPlugin(returnValue, 'sdpi_collection');
});

/**
   * 'sendValueToPlugin' is a wrapper to send some values to the plugin
   *
   * It is called with a value and the name of a property:
   *
   * sendValueToPlugin(<any value>), 'key-property')
   *
   * where 'key-property' is the property you listen for in your plugin's
   * 'sendToPlugin' events payload.
   *
   */

function sendValueToPlugin(value, prop) {
  if ($SD.connection && $SD.connection.readyState === 1) {
    const json = {
      action: $SD.actionInfo.action,
      event: 'sendToPlugin',
      context: $SD.uuid,
      payload: {
        [prop]: value,
        targetContext: $SD.actionInfo.context,
      },
    };

    $SD.connection.send(JSON.stringify(json));
  }
}

/** CREATE INTERACTIVE HTML-DOM
  * The 'prepareDOMElements' helper is called, to install events on all kinds of
  * elements (as seen e.g. in PISamples)
  * Elements can get clicked or act on their 'change' or 'input' event. (see at the top
  * of this file)
  * Messages are then processed using the 'handleSdpiItemChange' method below.
  * If you use common elements, you don't need to touch these helpers. Just take care
  * setting an 'id' on the element's input-control from which you want to get value(s).
  * These helpers allow you to quickly start experimenting and exchanging values with
  * your plugin.
  */

function prepareDOMElements(base) {
  const baseElement = base || document;
  Array.from(baseElement.querySelectorAll('.sdpi-item-value')).forEach(
    (el, i) => {
      const elementsToClick = [
        'BUTTON',
        'OL',
        'UL',
        'TABLE',
        'METER',
        'PROGRESS',
        'CANVAS',
      ].includes(el.tagName);
      const evt = elementsToClick ? 'onclick' : onchangeevt || 'onchange';

      /** Look for <input><span> combinations, where we consider the span as label for the input
              * we don't use `labels` for that, because a range could have 2 labels.
              */
      /** Just in case the found HTML element already has an input or change - event attached,
              * we clone it, and call it in the callback, right before the freshly attached event
             */
      const inputGroup = el.querySelectorAll('input + span');
      if (inputGroup.length === 2) {
        const offs = inputGroup[0].tagName === 'INPUT' ? 1 : 0;
        inputGroup[offs].textContent = inputGroup[1 - offs].value;
        inputGroup[1 - offs].oninput = () => {
          inputGroup[offs].textContent = inputGroup[1 - offs].value;
        };
      }
      /** We look for elements which have an 'clickable' attribute
             * we use these e.g. on an 'inputGroup' (<span><input type="range"><span>) to adjust the value of
             * the corresponding range-control
             */
      Array.from(el.querySelectorAll('.clickable')).forEach(
        (subel, subi) => {
          subel.onclick = (e) => {
            handleSdpiItemChange(e.target, subi);
          };
        },
      );
      /** Just in case the found HTML element already has an input or change - event attached,
             * we clone it, and call it in the callback, right before the freshly attached event
            */
      const cloneEvt = el[evt];
      el[evt] = (e) => {
        if (cloneEvt) cloneEvt();
        handleSdpiItemChange(e.target, i);
      };
    },
  );

  /**
     * You could add a 'label' to a textares, e.g. to show the number of charactes already typed
     * or contained in the textarea. This helper updates this label for you.
     */
  baseElement.querySelectorAll('textarea').forEach((e) => {
    const maxl = e.getAttribute('maxlength');
    e.targets = baseElement.querySelectorAll(`[for='${e.id}']`);
    if (e.targets.length) {
      const fn = () => {
        e.targets.forEach((target) => {
          target.textContent = maxl ? `${e.value.length}/${maxl}` : `${e.value.length}`;
        });
      };
      fn();
      e.onkeyup = fn;
    }
  });

  baseElement.querySelectorAll('[data-open-url').forEach((e) => {
    const value = e.getAttribute('data-open-url');
    if (value) {
      e.onclick = () => {
        let path;
        if (value.indexOf('http') !== 0) {
          path = document.location.href.split('/');
          path.pop();
          path.push(value.split('/').pop());
          path = path.join('/');
        } else {
          path = value;
        }
        $SD.api.openUrl($SD.uuid, path);
      };
    } else {
      console.warn(`${value} is not a supported url`);
    }
  });
}

// TODO: Refactor code below so that we can re-enable eslint
/* eslint-disable */

function handleSdpiItemChange(e, idx) {
  /** Following items are containers, so we won't handle clicks on them */

  if (['OL', 'UL', 'TABLE'].includes(e.tagName)) {
    return;
  }

  /** SPANS are used inside a control as 'labels'
     * If a SPAN element calls this function, it has a class of 'clickable' set and is thereby handled as
     * clickable label.
     */

  if (e.tagName === 'SPAN') {
    const inp = e.parentNode.querySelector('input');
    let tmpValue;

    // if there's no attribute set for the span, try to see, if there's a value in the textContent
    // and use it as value
    if (!e.hasAttribute('value')) {
      tmpValue = Number(e.textContent);
      if (typeof tmpValue === 'number' && tmpValue !== null) {
        e.setAttribute('value', 0 + tmpValue); // this is ugly, but setting a value of 0 on a span doesn't do anything
        e.value = tmpValue;
      }
    } else {
      tmpValue = Number(e.getAttribute('value'));
    }

    if (inp && tmpValue !== undefined) {
      inp.value = tmpValue;
    } else return;
  }

  const selectedElements = [];
  const isList = ['LI', 'OL', 'UL', 'DL', 'TD'].includes(e.tagName);
  const sdpiItem = e.closest('.sdpi-item');
  const sdpiItemGroup = e.closest('.sdpi-item-group');
  let sdpiItemChildren = isList
    ? sdpiItem.querySelectorAll(e.tagName === 'LI' ? 'li' : 'td')
    : sdpiItem.querySelectorAll('.sdpi-item-child > input');

  if (isList) {
    const siv = e.closest('.sdpi-item-value');
    if (!siv.classList.contains('multi-select')) {
      sdpiItemChildren.forEach((child) => child.classList.remove('selected'));
    }
    if (!siv.classList.contains('no-select')) {
      e.classList.toggle('selected');
    }
  }

  if (sdpiItemChildren.length && ['radio', 'checkbox'].includes(sdpiItemChildren[0].type)) {
    e.setAttribute('_value', e.checked); // '_value' has priority over .value
  }
  if (sdpiItemGroup && !sdpiItemChildren.length) {
    for (const x of ['input', 'meter', 'progress']) {
      sdpiItemChildren = sdpiItemGroup.querySelectorAll(x);
      if (sdpiItemChildren.length) break;
    }
  }

  if (e.selectedIndex !== undefined) {
    if (e.tagName === 'SELECT') {
      sdpiItemChildren.forEach((ec, i) => {
        selectedElements.push({ [ec.id]: ec.value });
      });
    }
    idx = e.selectedIndex;
  } else {
    sdpiItemChildren.forEach((ec, i) => {
      if (ec.classList.contains('selected')) {
        selectedElements.push(ec.textContent);
      }
      if (ec === e) {
        idx = i;
        selectedElements.push(ec.value);
      }
    });
  }

  const returnValue = {
    key: e.id && e.id.charAt(0) !== '_' ? e.id : sdpiItem.id,
    value: isList
      ? e.textContent
      : e.hasAttribute('_value')
        ? e.getAttribute('_value')
        : e.value
          ? e.type === 'file'
            ? decodeURIComponent(e.value.replace(/^C:\\fakepath\\/, ''))
            : e.value
          : e.getAttribute('value'),
    group: sdpiItemGroup ? sdpiItemGroup.id : false,
    index: idx,
    selection: selectedElements,
    checked: e.checked,
  };

  /** Just simulate the original file-selector:
     * If there's an element of class '.sdpi-file-info'
     * show the filename there
     */
  if (e.type === 'file') {
    const info = sdpiItem.querySelector('.sdpi-file-info');
    if (info) {
      const s = returnValue.value.split('/').pop();
      info.textContent = s.length > 28
        ? `${s.substr(0, 10)
        }...${
          s.substr(s.length - 10, s.length)}`
        : s;
    }
  }

  $SD.emit('piDataChanged', returnValue);
}

/**
 * This is a quick and simple way to localize elements and labels in the Property
 * Inspector's UI without touching their values.
 * It uses a quick 'lox()' function, which reads the strings from a global
 * variable 'localizedStrings' (in 'common.js')
 */

// eslint-disable-next-line no-unused-vars
function localizeUI() {
  const el = document.querySelector('.sdpi-wrapper') || document;
  let t;
  Array.from(el.querySelectorAll('sdpi-item-label')).forEach((e) => {
    t = e.textContent.lox();
    if (e !== t) {
      e.innerHTML = e.innerHTML.replace(e.textContent, t);
    }
  });
  Array.from(el.querySelectorAll('*:not(script)')).forEach((e) => {
    if (
      e.childNodes
            && e.childNodes.length > 0
            && e.childNodes[0].nodeValue
            && typeof e.childNodes[0].nodeValue === 'string'
    ) {
      t = e.childNodes[0].nodeValue.lox();
      if (e.childNodes[0].nodeValue !== t) {
        e.childNodes[0].nodeValue = t;
      }
    }
  });
}

/**
 *
 * Some more (de-) initialization helpers
 *
 */

document.addEventListener('DOMContentLoaded', () => {
  document.body.classList.add(navigator.userAgent.includes('Mac') ? 'mac' : 'win');
  prepareDOMElements();
  $SD.on('localizationLoaded', (language) => {
    localizeUI();
  });
});

/** the beforeunload event is fired, right before the PI will remove all nodes */
window.addEventListener('beforeunload', (e) => {
  e.preventDefault();
  sendValueToPlugin('propertyInspectorWillDisappear', 'property_inspector');
  // Don't set a returnValue to the event, otherwise Chromium with throw an error.  // e.returnValue = '';
});
