/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const LookAction = require('../../js/actions/LookAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new LookAction(apiManager);
    done();
  });

  // *** Tests ***

  it('Should initialize a new Look Action instance', () => {
    assert(action != null);
  });

  it('Should get looks when `pollForData` is called and should call `sendToPropertyInspector`', async () => {
    const spyOnGet = sinon.spy(apiManager, 'getLooks');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.LOOK_ACTION_ID);

    assert(spyOnGet.called);
    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableLooks.length === 2);
  });

  it('Should send null for `availableLooks` to property inspector when error is thrown', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getLooks = () => {
      throw new Error();
    };

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.LOOK_ACTION_ID);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableLooks === null);
  });

  it('Should fire trigger look when the `performAction` event runs', () => {
    const json = FakeStreamDeck.generateJSON(Constants.LOOK_ACTION_ID, { lookSelector: 'foobar' });
    const spytriggerLook = sinon.spy(apiManager, 'triggerLook');

    action.performAction(json.payload.settings);

    assert(spytriggerLook.calledWith('foobar'));
  });

  it('Should throw an error if `lookSelector` is not defined when performing action', async () => {
    await assert.rejects(action.performAction({}), Error);
  });
}

module.exports = test;
