/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const MacroAction = require('../../js/actions/MacroAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new MacroAction(apiManager);
    done();
  });

  // *** Tests ***

  it('Should initialize a new Macro Action instance', () => {
    assert(new MacroAction(new FakeAPIManager()) != null);
  });

  it('Should get macros when `pollForData` is called and should call `sendToPropertyInspector`', async () => {
    const spyOnGet = sinon.spy(apiManager, 'getMacros');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.MACRO_ACTION_ID);

    assert(spyOnGet.called);
    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableMacros.length === 2);
  });

  it('Should send null for `availableMacros` to property inspector when error is thrown', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getMacros = () => {
      throw new Error();
    };

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.MACRO_ACTION_ID);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableMacros === null);
  });

  it('Should fire trigger macro when the `performAction` event runs', () => {
    const json = FakeStreamDeck.generateJSON(Constants.MACRO_ACTION_ID, { macroSelector: 'foobar' });
    const spytriggerMacro = sinon.spy(apiManager, 'triggerMacro');

    action.performAction(json.payload.settings);

    assert(spytriggerMacro.calledWith('foobar'));
  });

  it('Should throw an error if `macroSelector` is not defined when performing action', async () => {
    await assert.rejects(action.performAction({}), Error);
  });
}

module.exports = test;
