/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const SlideAction = require('../../js/actions/SlideAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new SlideAction(apiManager);
    done();
  });

  // *** Tests ***

  it('Should initialize a new SlideAction instance', () => {
    assert(action != null);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings.modeSelector === 'TriggerNextSlide');
    assert(settings['icon-state'] === 'next');
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      modeSelector: 'TriggerNextSlide',
      'icon-state': 'next',
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings.modeSelector === 'TriggerNextSlide');
    assert(settings['icon-state'] === 'next');
  });

  it('Should fire next slide action', () => {
    const json = FakeStreamDeck.generateJSON(Constants.SLIDE_ACTION_ID, { modeSelector: 'TriggerNextSlide' });
    const spyOnTriggerNextSlide = sinon.spy(apiManager, 'triggerNextSlide');

    action.onKeyUp(json);

    assert(spyOnTriggerNextSlide.called);
  });

  it('Should fire previous slide action', () => {
    const json = FakeStreamDeck.generateJSON(Constants.SLIDE_ACTION_ID, { modeSelector: 'TriggerPreviousSlide' });
    const spyOnTriggerPreviousSlide = sinon.spy(apiManager, 'triggerPreviousSlide');

    action.onKeyUp(json);

    assert(spyOnTriggerPreviousSlide.called);
  });

  it('Should throw an error if `modeSelector` is not defined when performing action', async () => {
    await assert.rejects(action.performAction({}), Error);
  });

  it('Should throw an error if `modeSelector` is invalid when performing action', async () => {
    await assert.rejects(action.performAction({ modeSelector: 'Merp' }), Error);
  });
}

module.exports = test;
