/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const VideoInputAction = require('../../js/actions/VideoInputAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new VideoInputAction(apiManager);
    done();
  });

  // *** Tests ***

  it('Should create a new VideoInputAction instance', () => {
    assert(action != null);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings['icon-state'] === 'trigger');
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      'icon-state': 'trigger',
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings['icon-state'] === 'trigger');
  });

  it('Should trigger the selected video input when the `performAction` event fires', async () => {
    const json = FakeStreamDeck.generateJSON(Constants.VIDEO_INPUT_ACTION_ID, { videoInputSelector: '8A053257-929E-4A32-8784-079471A396A1' });
    const spyOnTriggerVideoInput = sinon.spy(apiManager, 'triggerVideoInput');

    await action.performAction(json.payload.settings);

    assert(spyOnTriggerVideoInput.calledWith('8A053257-929E-4A32-8784-079471A396A1'));
  });

  it('Should get video inputs when `pollForData` is called and should call `sendToPropertyInspector`', async () => {
    const spyOnGet = sinon.spy(apiManager, 'getVideoInputs');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.VIDEO_INPUT_ACTION_ID);

    assert(spyOnGet.called);
    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableVideoInputs.length === 2);
  });

  it('Should send null for `availableVideoInputs` to property inspector when error is thrown', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getVideoInputs = () => {
      throw new Error();
    };

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.VIDEO_INPUT_ACTION_ID);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableVideoInputs === null);
  });

  it('Should throw an error if `videoInputSelector` is not defined when performing action', async () => {
    await assert.rejects(action.performAction({}), Error);
  });
}

module.exports = test;
