const FakeActionInfo = require('./FakeActionInfo');
const FakeApi = require('./FakeApi');

class FakeStreamDeck {
  constructor() {
    this.api = new FakeApi();
    this.actionInfo = new FakeActionInfo();
    this.on = () => {};
  }

  static #generateID = (len = 10, num = Number.MAX_SAFE_INTEGER) => Array.from(new Array(len))
    .map(() => Math.floor(Math.random() * num).toString(16))
    .join('-');

  /**
   * @param {string} action - The action ID to insert in the generated JSON.
   * @param {Object} settings - The settings to insert in the generated JSON.
   * @returns A an object that simulates the JSON that StreamDeck would pass into events.
   */
  static generateJSON(action, settings = {}) {
    const json = {
      action,
      context: FakeStreamDeck.#generateID(),
      device: '0964A314E2883277496AB02741B24192',
      event: '',
      payload: {
        state: '',
        userDesiredState: '',
        coordinates: { column: 3, row: 2 },
        isInMultiAction: false,
        settings,
      },
    };

    return json;
  }
}

module.exports = FakeStreamDeck;
