// eslint-disable-next-line import/no-extraneous-dependencies
import { rest } from 'msw';

const testBaseURL = 'http://localhost:8080';
const transportLayers = ['presentation', 'announcment', 'audio'];
const clearLayers = ['audio', 'props', 'messages', 'announcements', 'slide', 'media', 'video_input'];

const handlers = [

  // *** Audio Action ***

  rest.get(`${testBaseURL}/v1/audio/playlists`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: {
          uuid: 'ED59503B-394D-4D5D-B0B0-5AE80D0BA025',
          name: 'Playlist Folder',
          index: 0,
        },
        type: 'group',
        children: [
          {
            id: {
              uuid: 'C49A250A-3E32-4C26-A220-46D6596426B5',
              name: 'Playlist 1',
              index: 0,
            },
            type: 'playlist',
            children: [],
          },
          {
            id: {
              uuid: 'D8C5D151-45C9-4CFF-B1F9-74F3B4B5173B',
              name: 'Playlist 2',
              index: 1,
            },
            type: 'playlist',
            children: [],
          },
        ],
      },
    ]));
  }),

  rest.get(`${testBaseURL}/v1/audio/playlist/:playlistID`, (req, res, ctx) => {
    const start = req.url.searchParams.get('start');
    if (start !== '0') {
      return res(ctx.status(400));
    }
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json(
      {
        id: {
          uuid: 'B238C071-3628-4204-BB19-DDE76DC57CE7',
          name: 'Playlist 3',
          index: 2,
        },
        items: [
          {
            id: {
              uuid: '94A2D4EA-EEA5-48A4-AF6F-850D700C1830',
              name: 'Song 1',
              index: 0,
            },
            type: 'audio',
            artist: 'unknown',
            duration: 123,
          },
          {
            id: {
              uuid: '8F40D44E-4018-4552-8803-60DC788DB1CF',
              name: 'Amazing Grace',
              index: 1,
            },
            type: 'audio',
            artist: 'John Newton',
            duration: 110,
          },
        ],
      },
    ));
  }),

  rest.get(`${testBaseURL}/v1/audio/playlist/:playlistID/:itemID/trigger`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    if (req.params.playlistID !== 'PLAYLIST_WITH_ITEMS_ID' && req.params.itemID !== 'ITEM_ID') {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  rest.get(`${testBaseURL}/v1/audio/playlist/:playlistID/trigger`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    if (req.params.playlistID !== 'PLAYLIST_ID') {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Stage Action ***

  rest.get(`${testBaseURL}/v1/stage/screens`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: {
          name: 'Stage Left',
          index: 0,
          uuid: 'BB6E8C3A-8182-4818-8D57-38B1D85333A9',
        },
      },
      {
        id: {
          name: 'Stage Right',
          index: 1,
          uuid: 'D2BAE32A-CDE2-4166-BA5C-2276D51EEFB8',
        },
      },
    ]));
  }),
  rest.get(`${testBaseURL}/v1/stage/layouts`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: {
          name: 'Current/Next Stacked - Music',
          index: 0,
          uuid: '4D542943-6CF2-4BEE-A625-7C37B79D8EEF',
        },
      },
      {
        id: {
          name: 'Main Screen Preview',
          index: 1,
          uiuid: 'D2BAE32A-CDE2-4166-BA5C-2276D51EEFB8',
        },
      },
    ]));
  }),
  rest.put(`${testBaseURL}/v1/stage/layout_map`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    if (req.headers.headers['content-type'] !== 'application/json') {
      return res(ctx.status(400));
    }
    return res(ctx.status(204));
  }),
  rest.put(`${testBaseURL}/v1/stage/message`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    if (req.headers.headers['content-type'] !== 'application/json') {
      return res(ctx.status(400));
    }
    return res(ctx.status(204));
  }),
  rest.delete(`${testBaseURL}/v1/stage/message`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Slide Action ***

  rest.get(`${testBaseURL}/v1/trigger/next`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/trigger/previous`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Find My Mouse Action ***

  rest.get(`${testBaseURL}/v1/find_my_mouse`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Capture Action ***

  rest.get(`${testBaseURL}/v1/capture/start`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/capture/stop`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Timer Action ***

  rest.get(`${testBaseURL}/v1/timers`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: {
          uuid: '11111111-C4B2-44F7-A55D-4CC913BB8A5D',
          name: 'Timer One',
          index: 0,
        },
        allows_overrun: true,
        countdown: {
          duration: 63242630,
        },
      },
      {
        id: {
          uuid: '22222222-C4B2-44F7-A55D-4CC913BB8A5D',
          name: 'Timer Two',
          index: 1,
        },
        allows_overrun: false,
        countdown: {
          duration: -48712412,
        },
      },
    ]));
  }),
  rest.get(`${testBaseURL}/v1/timer/:timerID/start`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.put(`${testBaseURL}/v1/timer/:timerID/start`, (req, res, ctx) => {
    const configuration = req.body;
    if (typeof configuration.id === 'undefined' || typeof configuration.allows_overrun === 'undefined') {
      return res(ctx.status(400));
    }
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/timer/:timerID/stop`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.put(`${testBaseURL}/v1/timer/:timerID/stop`, (req, res, ctx) => {
    const configuration = req.body;
    if (typeof configuration.id === 'undefined' || typeof configuration.allows_overrun === 'undefined') {
      return res(ctx.status(400));
    }
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/timer/:timerID/reset`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/timer/:timerID/increment/:amount`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Look Action ***

  rest.get(`${testBaseURL}/v1/looks`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: {
          uuid: '942C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
          index: 0,
          name: 'First Look',
        },
        screens: [
          {
            video_input: true,
            media: true,
            slide: true,
            props: true,
            messages: true,
            presentation: '42C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
            announcements: '42C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
            mask: '42C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
          },
        ],
      },
      {
        id: {
          uuid: '942C3FC3-XXXX-YYYY-A55D-4CC913BB8A00',
          index: 1,
          name: 'Second Look',
        },
        screens: [
          {
            video_input: true,
            media: true,
            slide: true,
            props: true,
            messages: true,
            presentation: '42C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
            announcements: '42C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
            mask: '42C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
          },
        ],
      },
    ]));
  }),
  rest.get(`${testBaseURL}/v1/look/*/trigger`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Macro Action ***

  rest.get(`${testBaseURL}/v1/macros`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: {
          uuid: '942C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
          index: 0,
          name: 'First Macro',
        },
        color: {
          red: 0,
          green: 0.54,
          blue: 0.87,
          alpha: 1,
        },
      },
      {
        id: {
          uuid: '942C3FC3-XXXX-YYYY-A55D-4CC913BB8A00',
          index: 1,
          name: 'Second Macro',
        },
        color: {
          red: 0,
          green: 0.54,
          blue: 0.87,
          alpha: 1,
        },
      },
    ]));
  }),
  rest.get(`${testBaseURL}/v1/macro/*/trigger`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Transport Action ***

  rest.get(`${testBaseURL}/v1/transport/*/play`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }

    const pathComponents = req.url.toString().split('/');
    const layer = pathComponents.slice(-2, -1)[0];

    if (!transportLayers.includes(layer)) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/transport/*/pause`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }

    const pathComponents = req.url.toString().split('/');
    const layer = pathComponents.slice(-2, -1)[0];

    if (!transportLayers.includes(layer)) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/transport/*/skip_backward/*`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }

    const pathComponents = req.url.toString().split('/');
    const layer = pathComponents.slice(-3, -2)[0];
    const time = pathComponents.slice(-1)[0];

    if (!transportLayers.includes(layer)) {
      return res(ctx.status(404));
    }
    if (Number.isNaN(parseInt(time, 10))) {
      return res(ctx.status(400));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/transport/*/skip_forward/*`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }

    const pathComponents = req.url.toString().split('/');
    const layer = pathComponents.slice(-3, -2)[0];
    const time = pathComponents.slice(-1)[0];

    if (!transportLayers.includes(layer)) {
      return res(ctx.status(404));
    }
    if (Number.isNaN(parseInt(time, 10))) {
      return res(ctx.status(400));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/transport/*/go_to_end`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }

    const pathComponents = req.url.toString().split('/');
    const layer = pathComponents.slice(-2, -1)[0];
    const time = req.url.searchParams.get('time');

    if (!transportLayers.includes(layer)) {
      return res(ctx.status(404));
    }
    if (Number.isNaN(parseInt(time, 10))) {
      return res(ctx.status(400));
    }
    return res(ctx.status(204));
  }),

  // *** Prop Action ***

  rest.get(`${testBaseURL}/v1/props`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: {
          uuid: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
          name: 'Bravo Team',
          index: 0,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'CCB4C60D-6580-4E19-A3EB-512EA5D1E735',
          name: 'Something Else',
          index: 1,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'AB0BEFD0-63B6-42F2-A594-C85C78513F02',
          name: 'Prop',
          index: 2,
        },
        is_active: false,
      },
    ]));
  }),
  rest.get(`${testBaseURL}/v1/prop/*/trigger`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/prop/*/clear`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Group Action ***

  rest.get(`${testBaseURL}/v1/groups`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: {
          uuid: 'A64F5355-9094-4797-B4BE-DA8C005D8DCC',
          name: 'Verse 1',
          index: 0,
        },
        color: {
          red: 0.0,
          green: 0.46666666865348816,
          blue: 0.800000011920929,
          alpha: 1.0,
        },
      },
      {
        id: {
          uuid: 'CA83B9AE-3B3E-468A-AD93-A429C823E297',
          name: 'Verse 2',
          index: 1,
        },
        color: {
          red: 0.0,
          green: 0.3490196168422699,
          blue: 0.6000000238418579,
          alpha: 1.0,
        },
      },
    ]));
  }),
  rest.get(`${testBaseURL}/v1/group/*`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Clear Action ***

  rest.get(`${testBaseURL}/v1/clear/groups`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        id: { uuid: '5C2E860C-166A-49E4-BFD4-C02E4BE447D4', name: 'Clear All', index: 0 },
        icon: 'X',
        tint: {
          red: 0.7543836832046509, green: 0.049948517233133316, blue: 0.1292288452386856, alpha: 1,
        },
        layers: ['audio', 'audio', 'message', 'prop', 'announcement', 'presentation', 'media', 'video_input'],
        stop_timeline_announcements: true,
        stop_timeline_presentation: true,
      }, {
        id: { uuid: '71B1CDAB-0F5D-4541-B651-34F7BADF7BE9', name: 'Clear Presentation', index: 1 },
        icon: 'Slide',
        tint: {
          red: 0.9897401928901672, green: 0.5910230875015259, blue: 0.03556394577026367, alpha: 1,
        },
        layers: ['presentation'],
        stop_timeline_announcements: false,
        stop_timeline_presentation: true,
      }, {
        id: { uuid: 'BD2A81A9-5F38-4643-8CC3-052CC241BE75', name: 'Clear Props/Messages', index: 2 },
        icon: 'Cupcake',
        tint: {
          red: 0.9860475063323975, green: 0.00833342969417572, blue: 0.5019235014915466, alpha: 1,
        },
        layers: ['prop', 'message', 'presentation'],
        stop_timeline_announcements: true,
        stop_timeline_presentation: true,
      }, {
        id: { uuid: '921FC33D-3EAC-417A-BAC2-788F24056E65', name: 'Clear Announcements', index: 3 },
        icon: 'Target',
        tint: {
          red: 0.29833340644836426, green: 0.7534772157669067, blue: 0.2297917902469635, alpha: 1,
        },
        layers: ['announcement'],
        stop_timeline_announcements: true,
        stop_timeline_presentation: true,
      }, {
        id: { uuid: '420EA802-26B9-4D23-949C-B0A5F1BC7A39', name: 'Clear Audio', index: 4 },
        icon: 'Audio',
        tint: {
          red: 0.5, green: 0.5, blue: 0.5, alpha: 1,
        },
        layers: ['audio', 'audio'],
        stop_timeline_announcements: true,
        stop_timeline_presentation: true,
      },
    ]));
  }),
  rest.get(`${testBaseURL}/v1/clear/group/*/trigger`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/clear/layer/*`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }

    const pathComponents = req.url.toString().split('/');
    const layer = pathComponents.slice(-1)[0];

    if (!clearLayers.includes(layer)) {
      return res(ctx.status(404));
    }

    return res(ctx.status(204));
  }),

  // *** Timeline Action ***

  rest.get(`${testBaseURL}/v1/presentation/focused/timeline/play`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/presentation/focused/timeline/pause`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
  rest.get(`${testBaseURL}/v1/presentation/focused/timeline/rewind`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),

  // *** Video Input Action ***

  rest.get(`${testBaseURL}/v1/video_inputs`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(200), ctx.json([
      {
        uuid: '8A053257-929E-4A32-8784-079471A396A1',
        name: 'Input 1',
        index: 0,
      },
      {
        uuid: 'CA01AA33-ACAF-4E80-9CEE-A9EB0FBADDEA',
        name: 'Input 2: Input 2',
        index: 1,
      },
    ]));
  }),
  rest.get(`${testBaseURL}/v1/trigger/video_input/*`, (req, res, ctx) => {
    if (!global.linkEnabled) {
      return res(ctx.status(404));
    }
    return res(ctx.status(204));
  }),
];

export default handlers;
