/* eslint-disable import/no-extraneous-dependencies, no-console */
/* global describe, before, beforeEach, afterEach, after */
const { global } = require('@sinonjs/commons');

// If you extend a class in production code you'll need to require it here and make it a global otherwise your test will fail
global.ActionBase = require('../js/actions/ActionBase');
global.StreamingEndpoint = require('../js/StreamingEndpoint');
global.fetch = require('node-fetch/lib/index');

// Import fakes
const FakeStreamDeck = require('./fakes/FakeStreamDeck');
const FakeUtils = require('./fakes/FakeUtils');

// Import your test files
const ActionBaseTests = require('./actionTests/ActionBase.test');
const AudioActionTests = require('./actionTests/AudioAction.test');
const APIManagerTests = require('./APIManager.test');
const CaptureActionTests = require('./actionTests/CaptureAction.test');
const ClearActionTests = require('./actionTests/ClearAction.test');
const FindMyMouseActionTests = require('./actionTests/FindMyMouseAction.test');
const GroupActionTests = require('./actionTests/GroupAction.test');
const LookActionTests = require('./actionTests/LookAction.test');
const MacroActionTests = require('./actionTests/MacroAction.test');
const PropActionTests = require('./actionTests/PropAction.test');
const SlideActionTests = require('./actionTests/SlideAction.test');
const StageActionTests = require('./actionTests/StageAction.test');
const TimelineActionTests = require('./actionTests/TimelineAction.test');
const TimerActionTests = require('./actionTests/TimerAction.test');
const TransportActionTests = require('./actionTests/TransportAction.test');
const VideoInputActionTests = require('./actionTests/VideoInputAction.test');
const Constants = require('../js/constants');

// Describe the test group
// Describe the test
// Assert

// NOTE: If you attempt to run a test on a member and it says not found, remember to add it to the module at the base of the class file
//
// EXMAPLE:
// In file js/utils.js at the base since we are testing getLocalization here we need to export that function otherwise the test
// won't be able to see it
//
// if(typeof module !== 'undefined'){
//    module.exports = {
//      getLocalization
//    };
// }

// TO RUN TEST:
// Run `npm install --save-dev mocha` and then `npm test`

// If all the tests pass, then the console window won't get blown up with all of the
// console.log noise from the application
const originalLogFunction = console.log;
const originalWarnFunction = console.warn;
const originalErrorFunction = console.error;

let log;
let warn;
let error;

// eslint-disable-next-line prefer-arrow-callback, func-names
before(async function () {
  global.Headers = fetch.Headers;
  global.KEY_ICONS = Constants.KEY_ICONS;

  // eslint-disable-next-line import/extensions
  const serverModule = await import('./fakes/server.mjs');
  const server = serverModule.default;
  server.listen();
});

// eslint-disable-next-line prefer-arrow-callback, func-names
beforeEach(function (done) {
  global.$SD = new FakeStreamDeck();
  global.Utils = new FakeUtils();

  log = '';
  warn = '';
  error = '';

  console.log = (msg) => {
    log += `${msg}\n`;
  };

  console.warn = (msg) => {
    warn += `${msg}\n`;
  };

  console.error = (msg) => {
    error += `${msg}\n`;
  };

  done();
});

// If a test has failed though, then we'll dump the log out to the screen
// eslint-disable-next-line func-names
afterEach(async function () {
  console.log = originalLogFunction; // undo dummy log function
  console.warn = originalWarnFunction;
  console.error = originalErrorFunction;

  if (this.currentTest.state === 'failed') {
    console.log(log);
    console.warn(warn);
    console.error(error);
  }

  // eslint-disable-next-line import/extensions
  const serverModule = await import('./fakes/server.mjs');
  const server = serverModule.default;
  server.resetHandlers();
});

// eslint-disable-next-line prefer-arrow-callback, func-names
after(async function () {
  // eslint-disable-next-line import/extensions
  const serverModule = await import('./fakes/server.mjs');
  const server = serverModule.default;
  server.close();
});

describe('APIManager Tests', APIManagerTests.bind(this));

describe('Action Base Tests', ActionBaseTests.bind(this));
describe('Audio Action Tests', AudioActionTests.bind(this));
describe('Capture Action Tests', CaptureActionTests.bind(this));
describe('Clear Action Tests', ClearActionTests.bind(this));
describe('Find My Mouse Action Tests', FindMyMouseActionTests.bind(this));
describe('Group Action Tests', GroupActionTests.bind(this));
describe('Look Action Tests', LookActionTests.bind(this));
describe('Macro Action Tests', MacroActionTests.bind(this));
describe('Prop Action Tests', PropActionTests.bind(this));
describe('Slide Action Tests', SlideActionTests.bind(this));
describe('Stage Action Tests', StageActionTests.bind(this));
describe('Timeline Action Tests', TimelineActionTests.bind(this));
describe('Timer Action Tests', TimerActionTests.bind(this));
describe('Transport Action Tests', TransportActionTests.bind(this));
describe('Video Input Action Tests', VideoInputActionTests.bind(this));
